#!/usr/bin/env python3
import sys

function = "DistanceToIn"
shapes = ["Box", "Orb", "Trapezoid", "Trd1", "Parallelepiped", "Paraboloid", "Tube - no rmin no phi", "Tube - rmin and phi > PI"]
factors = ["ROOT", "Geant4", "Specialized", "Vectorized"]

def fetch(source, factor, shape, func):
    activated = False
    for line in source:
        line = line.split(",")
        if line[0] == shape:
            activated = True
            header = line
        if line[0] == func and activated:
            pos = header.index(factor)
            try:
                val = float(line[pos])
                return val # * 100
            except:
                return 0
    return 0

def print_factor(factor, source):
    values = []
    for shape in shapes:
        if factor == "Geant4" and function == "Contains":
            values += [fetch(source, factor, shape, "Inside")]
        else:
            values += [fetch(source, factor, shape, function)]
    print(",".join([str(x) for x in values]))


def main():
    with open(sys.argv[1]) as f:
        source = f.readlines()
        source = [x.strip() for x in source]

    print(function)
    print("{0},{1}".format(len(shapes), len(factors)))
    print(",".join(shapes))
    print(",".join(factors))

    for factor in factors:
        print_factor(factor, source)

    # print(source)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import subprocess, sys, re, math

# -----
npoints = 1024
nrep= 1000
target = "table.txt"
# ----

def sh(cmd):
    return subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT).decode("utf-8")

def run(cmd):
    return subprocess.call(cmd, shell=True)

def toBranch(br):
    sh("git checkout {0}".format(br))

def fetch(output, source, function):
    allmatches = []
    for line in str(output).split("\n"):
        if line.startswith(source):
            match = re.search("{0}: (\d+.\d+)".format(function), line)
            if match:
                allmatches += [match.group(1)]
    if len(allmatches) == 0:
        return None
    if len(allmatches) > 1:
        raise Exception("More than one matches: " + " ".join(allmatches))
    return allmatches[0]


def simpleTable(shape, out, f):
    # f.write(shape + "\n")
    functions = ["Inside", "Contains", "DistanceToIn", "DistanceToOut", "SafetyToIn", "SafetyToOut"]
    sources = ["ROOT", "USolids", "Unspecialized", "Specialized", "Vectorized", "Geant4"]

    f.write(shape + "," + ",".join(sources) + "\n")
    for function in functions:
        f.write(function + ",")
        for source in sources:
            val = fetch(out, source, function)
            if val is not None:
                f.write(val)
            f.write(",")
        f.write("\n")
    f.write("\n")

def runall(processor, f):
    sh("cp compile.sh compile-safe.sh")
    toBranch("master")
    run("./compile-safe.sh")

    # ===== Box
    out = sh("./build/BoxBenchmark -npoints {0} -nrep {1} -dx 5 -dy 10 -dz 15".format(npoints, nrep))
    processor("Box", out, f)

    # ===== Tube
    phi = 2*math.pi
    out = sh("./build/TubeBenchmark -npoints {0} -nrep {1} -rmin 0 -rmax 20 -dz 40 -sphi 0 -dphi {2}".format(npoints, nrep, phi))
    processor("Tube - no rmin no phi", out, f)

    phi = 2*math.pi
    out = sh("./build/TubeBenchmark -npoints {0} -nrep {1} -rmin 10 -rmax 20 -dz 40 -sphi 0 -dphi {2}".format(npoints, nrep, phi))
    processor("Tube - rmin no phi", out, f)

    phi = 3*math.pi/2
    out = sh("./build/TubeBenchmark -npoints {0} -nrep {1} -rmin 0 -rmax 20 -dz 40 -sphi 0 -dphi {2}".format(npoints, nrep, phi))
    processor("Tube - no rmin and phi > PI", out, f)

    phi = 3*math.pi/2
    out = sh("./build/TubeBenchmark -npoints {0} -nrep {1} -rmin 10 -rmax 20 -dz 40 -sphi 0 -dphi {2}".format(npoints, nrep, phi))
    processor("Tube - rmin and phi > PI", out, f)

    # ===== Parallelepiped
    run("./compileWithoutUSolids.sh")
    out = sh("./build/ParallelepipedBenchmark -npoints {0} -nrep {1} -dx 3 -dy 3 -dz 3 -alpha 14.9 -theta 39 -phi 3.22".format(npoints, nrep))
    processor("Parallelepiped", out, f)

    # ===== Trd
    toBranch("trd-development")
    run("./compile-safe.sh")
    out = sh("./build/TrdBenchmark -npoints {0} -nrep {1} -dx1 10 -dx2 20 -dy1 30 -dy2 30 -dz 10".format(npoints, nrep))
    processor("Trd1", out, f)

    # ===== Orb
    toBranch("raman/Orb")
    run("./compile-safe.sh")
    out = sh("./build/OrbBenchmark -npoints {0} -nrep {1} -r 3".format(npoints, nrep))
    processor("Orb", out, f)

    # ===== Trapezoid
    toBranch("trap-SOAPlanes")
    run("./compile-safe.sh")
    out = sh("./build/TrapezoidBenchmarkScript -npoints {0} -nrep {1} -dz 15 -p1x -2 -p2x 2 -p3x -3 -p4x 3 -p5x -4 -p6x 4 -p7x -6 -p8x 6 -p1y -5 -p2y -5 -p3y 5 -p4y 5 -p5y -10 -p6y -10 -p7y 10 -p8y 10".format(npoints, nrep))
    processor("Trapezoid", out, f)

    # Back to master
    toBranch("master")
    
def main():
    f = open(target, "w")
    root = sh("git rev-parse --show-toplevel")
    root = root.strip()
    
    runall(simpleTable, f)

    f.close()
    sys.exit(0)

if __name__ == '__main__':
	main()




from Dataset import Dataset
import csv

def readPerformanceData(index, filename, datasets):
    """ Reads performance data from file <filename>, stores information
        into the non-empty <datasets> tuple.  
        The <index> will be used as the x-value in the plots.

        Example Usage: see scripts/plotNormalizedEvolution.py
    """
    ok = True
    try:
        file = open(filename, "rt")
    except:
        print "probs reading file?  file=<%s>" % filename
        ok=False
        pass

    if ok:
        try:
            reader = csv.reader(file, delimiter=' ')
            for dirtyRow in reader :
                row = filter(len, dirtyRow)   # remove null elements from list
                #print 'read: <%s>' % row
                if row[0]=="#": continue        # skip comment lines
                if row[1]=="-.------": continue # skip non-existent algorithms
                #print "row[0]=",row[0]
                #print "row[1]=",row[1]
                #print "row[2]=",row[2]
                #print "row[3]=",row[3]
                #print "row[4] = <%s>" % row[4]

                #.. parse input elements
                datatype = row[0]
                perf = float(row[1])
                error = float(row[2])

                #.. parse shape / algorithm / implementation
                istart = 0
                iend = istart + datatype[istart:].find("-")
                shape = datatype[istart:iend]

                istart = iend+1
                iend = istart + datatype[istart:].find("-")
                algo = datatype[istart:iend]

                istart = iend+1
                impl = datatype[istart:]
                key = shape+algo+impl
                try:
                    datasets[key].addPoint(index,perf,error)
                except KeyError:
                    #print "Creating key=<%s>, index=%i, perf=%f, error=%f" % (key, index, perf, error)
                    datasets[key] = Dataset( shape, algo, impl )
                    datasets[key].addPoint(index,perf,error)

        finally:
            file.close()
            print "Commit=<%s> - %i datasets read." % (filename[0:7], len(datasets))



#!/usr/bin/env python
#
# File: average.py
# Purpose: reads an input data file, prints average and st.dev. for input data
#

import math
from ROOT import TCanvas, TH1F, gROOT
gROOT.SetBatch(1)

#def drawText(xpos,ypos,color,text) :
#  txtObj = TLatex(xpos,ypos,text)
#  txtObj.SetTextColor(color)
#  txtObj.SetNDC(1)
#  txtObj.SetTextAlign(11)
#  txtObj.SetTextSize(0.02)
#  txtObj.Draw()
#  return txtObj

#.. parse options
from optparse import OptionParser
usage = "usage: %prog [options] arg1 arg2"
parser = OptionParser(usage=usage)
parser.add_option("-f", "--file", dest="filename", help="input FILE with list of funds to compare", metavar="FILE")
parser.add_option("-v", "--verbose", dest="debug", help="verbose output", action="store_true")
#parser.add_option("-q", "--quiet", dest="debug", help="verbose output", action="store_false", default=False)
#parser.add_option("-y", "--ymin", dest="ymin", help="min value of Y-axis", type="float", default=0)
#parser.add_option("-Y", "--ymax", dest="ymax", help="max value of Y-axis", type="float", default=0)
parser.add_option("-p", "--makeplots", dest="plots", help="make plots", action="store_true", default=False)
(options, args) = parser.parse_args()

if options.debug:
  print "verbose =", options.debug

#.. Read data
if options.debug: print "Reading values from file", options.filename

nval=0
sum=0.
sum2=0.
xhigh=0.
xlow=9999.

datafile = open(options.filename)
#for line in open(options.filename).read().split('\n'):
for line in datafile.read().split('\n'):
  if line=='': continue
  if line=='-.------': continue
  x = float(line)
  sum += x
  sum2 += x*x
  nval += 1
  if x > xhigh:
    xhigh = x
  if x < xlow:
    xlow = x

datafile.close()

#mean = sum/nval
#sigma = math.sqrt(nval*sum2-sum*sum)/(nval-1)
#print "\n Naive average: %f +/- %f (%i points, sigma/mean=%3.1f) xhigh=%f\n" % (mean, sigma, nval, sigma/mean*100, xhigh)

#.. always discard highest data point (outlier)
mean = 0.0
sigma = 0.0
quality = 0.0
if nval>0:
  sum -= xhigh
  sum2 -= xhigh*xhigh
  nval -= 1
  mean = sum/nval
  aux = nval*sum2 - sum*sum
  if aux<0.0:
    aux=0.0
  sigma = math.sqrt(aux)/(nval-1)

if mean>0:
  quality = sigma/mean*100

if options.debug is True:
  print "\n Improved average: %f +/- %f (%i points, sigma/mean=%3.1f)\n" % (mean, sigma, nval, quality)
else:
  print "%f %f %i %3.1f" % (mean, sigma, nval, quality)

#.. fill and save a histogram
if options.plots and xlow<xhigh:
  datafile = open(options.filename)
  hist = TH1F("hist","hist",100,0.9*xlow,1.1*xhigh)
  for line in datafile.read().split('\n'):
    if line=='': continue
    if line=='-.------': continue
    x = float(line)
    hist.Fill(x)

  datafile.close()

  c1 = TCanvas("c1","c1",400,400)
  hist.Draw()
  c1.SaveAs("last.png")



from ROOT import TGraph, TGraphErrors
#import sys
import csv
from datetime import date

class Datapoint:
    def __init__(self,x,y,dy):
        self._x = x
        self._y = y
        self._dy = dy

class Dataset:
    """ Reads performance data from a table: "version algo source timing uncertainty"
    """
    def addPoint(self,x,y,dy):
        self._data.append( Datapoint(x,y,dy) )
        if x < self._minkey : self._minkey = x
        if x > self._maxkey : self._maxkey = x
        if y < self._minvalue : self._minvalue = y
        if y > self._maxvalue : self._maxvalue = y

    def __init__(self, shape, algo, impl):
        """ constructor creates the data series"""
        self._shape = shape
        self._algo = algo
        self._impl = impl
        self._data = []
        self._minkey = 1.0e16
        self._maxkey = -1.0e16
        self._minvalue = 1.0e16
        self._maxvalue = -1.0e16

    def tgraph(self):
        """ returns data points in a TGraphErrors object."""
        if(len(self._data)==0):
            print "***** Dataset.tgraph(): ERROR: no data available for", self._algo+'_'+self._impl
            return TGraphErrors()
        result = TGraphErrors( len(self._data) )
        i = 0
        #for k in sorted(self._data.iterkeys()):
        for k in self._data:
            x = k._x
            y = k._y
            #if y < self._minvalue : self._minvalue = y
            #if y > self._maxvalue : self._maxvalue = y
            #print "raw %s %s %s: i=%d value=%f minVal=%f maxVal=%f" % (self._shape, self._algo, self._impl, i, k, self._minvalue, self._maxvalue)
            result.SetPoint( i, x, y )
            result.SetPointError( i, 0, k._dy )
            i = i + 1;
        return result


    def tgraphNorm(self):
        """ produces a timing series normalized to value=1.0 at the first data point"""
        if(len(self._data)==0):
            print "***** Dataset.tgraphNorm(): ERROR: no data available for", self._algo+'_'+self._impl
            return TGraph()
        result = TGraph( len(self._data) )
        normFactor = 1. / self._data[0]._y
        i = 0
        for k in self._data:
            normValue = k._y * normFactor
            if normValue < self._minvalue : self._minvalue = normValue
            if normValue > self._maxvalue : self._maxvalue = normValue
            #print "Normalizing %s: i=%d days=%d val=%f normFactor=%f stored=%f minVal=%f maxVal=%f" % (self._name, i, k, self.serie[k], normFactor, normValue, self._minvalue, self._maxvalue)
            result.SetPoint( i, k._x, normValue )
            i = i + 1;
        return result

    def tgraphNormError(self):
        """ returns a TGraphErrors object normalized to value=1.0 at the first data point"""
        if(len(self._data)==0):
            print "***** Dataset.tgraphNormError(): ERROR: no data available for", self._algo+'_'+self._impl
            return TGraphErrors()
        result = TGraphErrors( len(self._data) )
        normFactor = 1. / self._data[0]._y
        i = 0
        for k in self._data:
            normValue = k._y * normFactor
            if normValue < self._minvalue : self._minvalue = normValue
            if normValue > self._maxvalue : self._maxvalue = normValue
            #print "Normalizing %s: i=%d days=%d val=%f normFactor=%f stored=%f minVal=%f maxVal=%f" % (self._name, i, k, self.serie[k], normFactor, normValue, self._minvalue, self._maxvalue)
            result.SetPoint( i, k._x, normValue )
            result.SetPointError( i, 0, k._dy*normFactor )
            i = i + 1;
        return result


    def tgraphRatio(self, refDataset):
        """ produces timing ratio graphs"""
        if(len(self._data)==0):
            print "***** Dataset.tgraphNorm(): ERROR: no data available for", self._algo+'_'+self._impl
            return TGraphErrors()

        result = TGraph( len(self._data) )
        for i in range(len(self._data)):
            ratioValue = self._data[i]._y / refDataset._data[i]._y
            result.SetPoint( i, self._data[i]._x, ratioValue )
            if ratioValue < self._minvalue : self._minvalue = ratioValue
            if ratioValue > self._maxvalue : self._maxvalue = ratioValue
            #print "Normalizing %s: i=%d days=%d val=%f normFactor=%f stored=%f minVal=%f maxVal=%f" % (self._name, i, k, self.serie[k], normFactor, normValue, self._minvalue, self._maxvalue)
        return result

    def tgraphSpeedup(self, refDataset):
        """ produces speed-up graphs"""
        if(len(self._data)==0):
            print "***** Dataset.tgraphNorm(): ERROR: no data available for", self._algo+'_'+self._impl
            return TGraphErrors()

        result = TGraph( len(self._data) )
        #print "Speed-up: sizes: refDataset=%i self=%i" % (len(refDataset._data), len(self._data))
        for i in range(len(self._data)):
            speedup = refDataset._data[i]._y / self._data[i]._y
            result.SetPoint( i, self._data[i]._x, speedup )
            if speedup < self._minvalue : self._minvalue = speedup
            if speedup > self._maxvalue : self._maxvalue = speedup
        return result

    def getEarliestVersion(self):
        """ return earliest data point in a time series"""
        return self._minkey

    def getLatestVersion(self):
        """ return latest data point in a time series"""
        return self._maxkey

    def getLowestValue(self):
        """ return lowest data point in a time series"""
        if self._maxvalue<=0 and self._minvalue>=99999:
            print "ERROR in getLowestValue:",self._name,": make sure to call tgraphNorm() first!!!"
        return self._minvalue

    def getHighestValue(self):
        """ return highest value in a time series"""
        if self._maxvalue<=0 and self._minvalue>=99999:
            print "ERROR in getHighestValue:",self._name,": make sure to call tgraphNorm() first!!!"
        return self._maxvalue

    def getLastValue(self):
        return ( self.serie[self._maxkey] / self.serie[self._minkey] )

    def getFirstValue(self):
        return ( self.serie[self._minkey] / self.serie[self._minkey] )

### Testing

if __name__ == "__main__":

    ### these are the full series (without normalization)
    algoNames = ["inside", "distToIn", "safetyToIn", "contains", "distToOut", "safetyToOut"]
    implNames = ["root", "usolids", "unspec", "vector", "spec"]

    ### read in the data points from external file
    datasets = {}
    for algo in algoNames:
        for impl in implNames:
            datasets[ algo+impl ] = Dataset( algo, impl )

    ok = True
    try:
        file = open("trap-perf-hist.dat", "rt")
    except:
        print "probs reading file?  file=", filename
        ok=False
        pass

    if ok:
        try:
            reader = csv.reader(file, delimiter=' ')
            first=False
            for row in reader :
                if not first :
                    #print 'read: <%s>' % row
                    #print "row[0] = <%s>" % row[0]
                    #print "row[0][6:]=",row[0][6:]
                    #print "row[0][3:5]=",row[0][3:5]
                    #print "row[0][0:2]=",row[0][0:2]
                    vers = int(row[0])
                    algo = row[1]
                    impl = row[2]
                    perf = float(row[3])
                    error = float(row[4])
                    datasets[algo+impl].addPoint(vers,perf,error)
                    #print vers,algo,impl,perf,error
                else :
                    #.. just skip reading header line in file
                    #print "len(row)=",len(row),":",row
                    if len(row)>0 and row[0]=='Data': first=False
                    pass
        finally:
            file.close()
            print len(datasets),"datasets read."


#    #.. show how to loop over data
#    for key,val in datasets.iteritems():
#        print val._algo, val._impl, len(val._data),val._minkey,val._maxkey
#        for i in range(len(val._data)):
#            print "   ",val._data[i]._x,val._data[i]._y,val._data[i]._dy

    #.. Make plots
    from ROOT import TCanvas, gROOT, TGraphErrors, TPaveText, TLatex, TH1F, TProfile, TFile, gStyle

    ### Plot 1

    c1 = TCanvas('c1','c1', 900, 600 )
    c1.Divide(3,2)

    ### graphs contain the normalized datasets
    graphs = []
    for i in range(6): graphs.append([])  # 6 empty lists, one for each algo

    #.. separate the data into the 6 different algorithms
    for key,value in datasets.iteritems() :
        #print "Dataset %s: %d elements between %s and %s" % (key, len(value._data), value._minkey, value._maxkey )
        for ialgo in range(6):
            ilen = len(algoNames[ialgo])
            if key[0:ilen] == algoNames[ialgo]:
                normGraph = value.tgraphNorm()
                for impl in range(6):
                    icolor = impl+2
                    if icolor>=5: icolor+=1
                    if key[ilen:] == implNames[impl]:
                        normGraph.SetLineColor(icolor)
                        normGraph.SetMarkerStyle(20+icolor)
                        normGraph.SetMarkerColor(icolor)
                        break
                #graphs[ialgo].append( normGraph )
                graphs[ialgo].append(  )

    #.. now plot the 6 separate groups of datasets
    for ialgo in range(len(graphs)):
        ipad = ialgo + 1
        ymin = 1.1
        ymax = 0.9
        for igraph in range(len(graphs[ialgo])):
            tmp = graphs[ialgo][igraph]
            npts = tmp.GetN()
            x = tmp.GetX()
            y = tmp.GetY()
            for i in range(npts):
                #print "point %i: (%f; %f)" % (i,x[i],y[i])
                if y[i]<ymin: ymin = y[i]
                if y[i]>ymax: ymax = y[i]
            #print "ialgo=%i, igraph=%i, ymin=%f, ymax=%f" % (ialgo,igraph,ymin,ymax)

        thePad = c1.cd(ipad)
        thePad.SetGrid(1,1)
        frame1 = thePad.DrawFrame(0, 0.95*ymin, 10, 1.05*ymax)
        frame1.SetTitle(algoNames[ialgo])
        frame1.GetXaxis().SetTitle("Version")
        frame1.GetYaxis().SetTitle("Time [sec]")

        for igra in range(len(graphs[ialgo])):
                tmp = graphs[ialgo][igra]
                #tmp.SetLineColor(ialgo+1)
                if(tmp.GetN()>0): tmp.Draw("pl")

    c1.Update()
    c1.SaveAs("last.png")



#!/usr/bin/python
#
# File: plotSpeedups.py
#
from ROOT import TCanvas, gROOT, TGraphErrors, TPaveText, TLatex, TFile, gStyle
from PerfHistoryTools import readPerformanceData

### these are the full series

commits = ["68c48276","4e5928c1","019d29a0","b356bad1","2c007289",
           "298df455","a860eea3","e569cfb7","9e47e5fd","0877406d"]
#           "143d582e","b92b66ba","3af4b3f2","b9437a06","e4c54450","c2f9b8cc","ec7c80a5","1047750f","30635340"]
#commits = ["143d582e","b92b66ba","3af4b3f2","b9437a06","e4c54450","c2f9b8cc","ec7c80a5","1047750f","30635340"]
#commits   = ["c4154901", "1f673e0d", "458e6f08"]
shapes    = ["Trapezoid"]
algoNames = ["inside", "distToIn", "safeToIn", "contains", "distToOut", "safeToOut"]
implNames = ["vect", "unspec", "spec", "root", "usolids", "geant4"]
#implNames = ["spec", "vect", "cuda"]
refNames  = ["insidespec", "distToInspec","safeToInspec","containsspec","distToOutspec","safeToOutspec"]

### read in the data points from external files

#.. loop over available commits, filling perf data into datasets
datasets = {}
i = 0
while i < len(commits):
    commit = commits[i]
    filename = commit+"/"+commit+"-perf.dat"
    #print "commit: <%s> and file=<%s>" % (commit, filename)
    readPerformanceData( i+1, filename, datasets )
    i = i + 1

#.. show how to loop over data
#for key,val in datasets.iteritems():
#    print val._shape, val._algo, val._impl, len(val._data), val._minkey, val._maxkey
#    for i in range(len(val._data)):
#        print "   ",val._data[i]._x,val._data[i]._y,val._data[i]._dy


#.. Prepare plotting canvas
c1 = TCanvas('c1','c1', 900, 600 )
c1.Divide(3,2)

#.. loop over all shapes requested
for shname in shapes:
    shoffset = len(shname)

    ###.. define references for speedups
    refGraphs = []
    for name in refNames:
        for key,value in datasets.iteritems():
            if key==shname+name:
                #print "Dataset %s: %d datasets between %s and %s" % (key, len(value._data), value._minkey, value._maxkey )
                refGraphs.append( value )

    ### speedupGraphs will contain the normalized datasets
    speedupGraphs = []
    for i in range(6): speedupGraphs.append([])  # 6 empty lists for each shape, one list per algo

    #.. loop over perf data loaded into datasets, to separate the data into the 6 distinct algorithms
    for key,dset in datasets.iteritems() :
        if dset._shape != shname: continue
        algo = dset._algo
        impl = dset._impl
        #print "shname=%s - Dataset %s %s %s: %d elements between %s and %s" % (shname, dset._shape, algo, impl, len(dset._data), dset._minkey, dset._maxkey )

        #.. search perf data for each algorithm requested
        for ialgo in range(6):
            ilen = shoffset + len(algoNames[ialgo])
            if key[shoffset:ilen] == algoNames[ialgo]:
                #evoGraph = dset.tgraphNormError()
                speedupGraph = dset.tgraphSpeedup( refGraphs[ialgo] )

                #.. search perf data for each implementation requested
                for iimpl in range(6):
                    icolor = iimpl+1
                    if icolor>4: icolor+=1
                    #ilen2 = len(implNames[iimpl])
                    if key[ilen:] == implNames[iimpl]:
                        #print "found:",iimpl, implNames[iimpl]
                        speedupGraph.SetLineColor(icolor)
                        speedupGraph.SetMarkerStyle(20+iimpl)
                        speedupGraph.SetMarkerColor(icolor)
                        break
                speedupGraphs[ialgo].append( speedupGraph )

    #.. now plot the 6 separate groups of datasets
    for ialgo in range(len(speedupGraphs)):
        ipad = ialgo + 1

        #.. scan data for ymin,ymax scale adjustment
        ymin = 999999.
        ymax = -ymin
        for igraph in range(len(speedupGraphs[ialgo])):
            tmp = speedupGraphs[ialgo][igraph]
            npts = tmp.GetN()
            x = tmp.GetX()
            y = tmp.GetY()
            for i in range(npts):
                #print "point %i: (%f; %f)" % (i,x[i],y[i])
                if y[i]<ymin: ymin = y[i]
                if y[i]>ymax: ymax = y[i]
            #print "ialgo=%i, igraph=%i, ymin=%f, ymax=%f" % (ialgo,igraph,ymin,ymax)

        gPad = c1.cd(ipad)
        gPad.SetGrid(1,1)
        #gPad.SetLogx(1)
        #gPad.SetLogy(1)
        frame1 = gPad.DrawFrame(0, 0.01, 1+len(commits), 1.05*ymax)
        frame1.SetTitle(algoNames[ialgo])
        frame1.GetXaxis().SetTitle("N tracks")
        frame1.GetYaxis().SetTitle("Speed-up")

        for igra in range(len(speedupGraphs[ialgo])):
            tmp = speedupGraphs[ialgo][igra]
            #tmp.SetLineColor(ialgo+1)
            if(tmp.GetN()>0): tmp.Draw("ELP")

    c1.SetTitle("Speed-ups")
    c1.Update()
    c1.SaveAs("speedups-"+shname+".png")



# Script to generate translation and rotation specializations of placed volumes,

rotation = [0x1B1, 0x18E, 0x076, 0x16A, 0x155, 0x0AD, 0x0DC, 0x0E3, 0x11B,
            0x0A1, 0x10A, 0x046, 0x062, 0x054, 0x111, 0x200]
translation = ["translation::kGeneric", "translation::kIdentity"]

header_string = """\
/// @file TransformationSpecializations.icc
/// @author Script generated.

#ifndef VECGEOM_NO_SPECIALIZATION
"""

specialization_string = """\
  if (trans_code == {:s} && rot_code == {:#05x}) {{
    return VolumeType::template Create<{:s}, {:#05x}>(
      logical_volume, transformation,
#ifdef VECCORE_CUDA
           id, copy_no, child_id,
#endif
      placement);
  }}
"""

generic_string = """\
#endif // No specialization

  return VolumeType::template Create<translation::kGeneric, rotation::kGeneric>(
      logical_volume, transformation,
#ifdef VECCORE_CUDA
           id, copy_no, child_id,
#endif
      placement);\
"""

output_file = open("../VecGeom/management/TransformationSpecializations.icc", "w")

output_file.write(header_string)
for r in rotation:
  for t in translation:
    output_file.write(specialization_string.format(t, r, t, r))
output_file.write(generic_string)
output_file.close()



import matplotlib.pyplot as plt, csv, sys, numpy as np

if len(sys.argv) < 2:
  print("Please provide path to input data file.")
  sys.exit()

read = {}
labels = {}
cols = 0
rows = 0
with open(sys.argv[1]) as datafile:
  reader = csv.reader(datafile)
  headers = reader.__next__()
  cols = len(headers)
  for i in range(cols):
    read[i] = []
    labels[i] = headers[i]
  for row in reader:
    rows += 1
    for i in range(cols):
      read[i].append(row[i])
data = {}
for i in range(cols):
  data[labels[i]] = read[i]

def plot_point_count():
  for method in ["Inside", "DistanceToIn", "SafetyToIn"]:
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.set_xscale("log")
    point_count = []
    elapsed_scalar = []
    elapsed_vector = []
    elapsed_cuda = []
    elapsed_cudamem = []
    for i in range(rows):
      if data["method"][i] == method:
        if data["library"][i] == "Specialized":
          point_count.append(int(data["points"][i]))
          elapsed_scalar.append(float(data["elapsed"][i])/point_count[-1])
        if data["library"][i] == "Vectorized":
          elapsed_vector.append(elapsed_scalar[-1]/(float(data["elapsed"][i])
                                /point_count[-1]))
        if data["library"][i] == "CUDA":
          elapsed_cuda.append(elapsed_scalar[-1]/(float(data["elapsed"][i])
                              /point_count[-1]))
        if data["library"][i] == "CUDAMemory":
          elapsed_cudamem.append(elapsed_scalar[-1]/(float(data["elapsed"][i])
                                 /point_count[-1]))
    ax.plot(point_count, np.ones(len(point_count), dtype=np.float), "--b",
            label="Scalar")
    ax.plot(point_count, elapsed_vector, "-xr", ms=5,
            label="CPU SIMD instructions")
    ax.plot(point_count, elapsed_cuda, "-xg", ms=5,
            label="CUDA, best case memory overhead")
    ax.plot(point_count, elapsed_cudamem, "--", color=[0, 1, 0], ms=5,
            label="CUDA, worst case memory overhead")
    ax.set_xlim(point_count[0], point_count[-1])
    plt.xticks(point_count, [str(x) for x in point_count])
    ybegin, yend = ax.get_ylim()
    ax.set_yticks(np.arange(0, yend, 1.0))
    ax.set_title("Performance of %s algorithm for tubes" % method)
    ax.set_xlabel("Input length")
    ax.set_ylabel("Speedup")
    ax.legend(loc="upper left")
    fig.savefig("%s.pdf" % method)

plot_point_count()


#!/usr/bin/env python

"""
Example of creating a radar chart (a.k.a. a spider or star chart) [1]_.

Although this example allows a frame of either 'circle' or 'polygon', polygon
frames don't have proper gridlines (the lines are circles instead of polygons).
It's possible to get a polygon grid by setting GRIDLINE_INTERPOLATION_STEPS in
matplotlib.axis to the desired number of vertices, but the orientation of the
polygon is not aligned with the radial axes.

.. [1] http://en.wikipedia.org/wiki/Radar_chart
"""
import numpy as np
import sys

import matplotlib.pyplot as plt
import matplotlib as mpl
from matplotlib.patches import Polygon
from matplotlib.path import Path
from matplotlib.spines import Spine
from matplotlib.projections.polar import PolarAxes
from matplotlib.projections import register_projection


def radar_factory(num_vars, frame='circle'):
    """Create a radar chart with `num_vars` axes.

    This function creates a RadarAxes projection and registers it.

    Parameters
    ----------
    num_vars : int
        Number of variables for radar chart.
    frame : {'circle' | 'polygon'}
        Shape of frame surrounding axes.

    """
    # calculate evenly-spaced axis angles
    theta = 2*np.pi * np.linspace(0, 1-1./num_vars, num_vars)
    # rotate theta such that the first axis is at the top
    theta += np.pi/2

    def draw_poly_patch(self):
        verts = unit_poly_verts(theta)
        return plt.Polygon(verts, closed=True, edgecolor='k')

    def draw_circle_patch(self):
        # unit circle centered on (0.5, 0.5)
        return plt.Circle((0.5, 0.5), 0.5)

    patch_dict = {'polygon': draw_poly_patch, 'circle': draw_circle_patch}
    if frame not in patch_dict:
        raise ValueError('unknown value for `frame`: %s' % frame)

    class RadarAxes(PolarAxes):

        name = 'radar'
        # use 1 line segment to connect specified points
        RESOLUTION = 1
        # define draw_frame method
        draw_patch = patch_dict[frame]

        # def setlinestyle(self, ls):

        def fill(self, *args, **kwargs):
            """Override fill so that line is closed by default"""
            closed = kwargs.pop('closed', True)
            return super(RadarAxes, self).fill(closed=closed, *args, **kwargs)

        def plot(self, *args, **kwargs):
            """Override plot so that line is closed by default"""
            lines = super(RadarAxes, self).plot(*args, **kwargs)
            for line in lines:
                self._close_line(line)

        def _close_line(self, line):
            x, y = line.get_data()
            # FIXME: markers at x[0], y[0] get doubled-up
            if x[0] != x[-1]:
                x = np.concatenate((x, [x[0]]))
                y = np.concatenate((y, [y[0]]))
                line.set_data(x, y)

        def set_varlabels(self, labels):
            self.set_thetagrids(theta * 180/np.pi, labels)

        def _gen_axes_patch(self):
            return self.draw_patch()

        def _gen_axes_spines(self):
            if frame == 'circle':
                return PolarAxes._gen_axes_spines(self)
            # The following is a hack to get the spines (i.e. the axes frame)
            # to draw correctly for a polygon frame.

            # spine_type must be 'left', 'right', 'top', 'bottom', or `circle`.
            spine_type = 'circle'
            verts = unit_poly_verts(theta)
            # close off polygon by repeating first vertex
            verts.append(verts[0])
            path = Path(verts)

            spine = Spine(self, spine_type, path, linewidth=2)
            spine.set_transform(self.transAxes)
            return {'polar': spine}

    register_projection(RadarAxes)
    return theta


def unit_poly_verts(theta):
    """Return vertices of polygon for subplot axes.

    This polygon is circumscribed by a unit circle centered at (0.5, 0.5)
    """
    x0, y0, r = [0.5] * 3
    verts = [(r*np.cos(t) + x0, r*np.sin(t) + y0) for t in theta]
    return verts


if __name__ == '__main__':
    # Read data file
    with open(sys.argv[1]) as f:
        caption = f.readline().strip()

        line = f.readline()
        N, M = line.split(",")
        N = int(N)
        M = int(M)

        line = f.readline().strip()
        columns = line.split(",")
        assert len(columns) == N
        data = {}
        data["column names"] = columns
        data[caption] = []

        line = f.readline().strip()
        factors = line.split(",")
        assert len(factors) == M

        # For each factor
        for m in range(0,M):
            line = f.readline().strip()
            line = line.split(",")
            line = [float(x) for x in line]
            print(line)
            assert len(line) == N
            data[caption].append(line)

        data[caption].append( [0] * N)

        print("Got N = {0}, M = {1}".format(N, M))
        print(columns)
        # sys.exit(1)
    
    # N = N+1
    mpl.rcParams['axes.linewidth'] = 2
    # mpl.rcParams['lines.linewidth'] = 2
    theta = radar_factory(N, frame='polygon')

    # data = get_data()
    spoke_labels = data.pop('column names')

    fig = plt.figure(figsize=(10, 10), facecolor="white")
    fig.subplots_adjust(wspace=0.25, hspace=0.20, top=0.85, bottom=0.05)

    colors = ['#C7F464', '#C44D58', '#4ECDC4', '#556270', 'c', 'k']
    while len(colors) > len(factors)+1:
        colors.pop()
    print(colors)
    # Plot the four cases from the example data on separate axes
    for n, title in enumerate(data.keys()):
        ax = fig.add_subplot(1, 1, n+1, projection='radar')
        # ax = fig.add_subplot(111, projection='radar')
        plt.rgrids([0.2, 0.4, 0.6, 0.8])
        ax.set_title(title, weight='bold', size='medium', position=(0.5, 1.1),
                     horizontalalignment='center', verticalalignment='center')
        for d, color in zip(data[title], colors):
            ax.plot(theta, d, color=color, linestyle="None")
            p = ax.fill(theta, d, facecolor=color, linestyle="solid", linewidth="2", edgecolor="#000000")
            p = p[0]
            print(type(p))
        ax.set_varlabels(spoke_labels)

    # add legend relative to top-left plot
    plt.subplot(1, 1, 1)
    labels = tuple(factors)

    # labels = ('Factor 1', 'Factor 2', 'Factor 3', 'Factor 4', 'Factor 5')
    legend = plt.legend(labels, loc=(0.9, .95), labelspacing=0.1, shadow=False)
    plt.setp(legend.get_texts(), fontsize='small')

    # plt.figtext(0.5, 0.965, '5-Factor Solution Profiles Across Four Scenarios',
    #             ha='center', color='black', weight='bold', size='large')
    plt.show()
